@@ -30,8 +30,11 @@
 #include <syslog.h>
 #include <time.h>
 #include <sys/utsname.h>
+#include <sys/types.h>
+#include <signal.h>
 
 #include <glib/gi18n.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include "gdm.h"
@@ -40,6 +43,7 @@
 #include "gdmconfig.h"
 
 gint gdm_timed_delay = 0;
+static Atom AT_SPI_IOR;
 
 /*
  * Some slaves want to send output to syslog and others (such as 
@@ -270,7 +274,7 @@ gdm_common_setup_blinking (void)
 
 	sid = g_signal_lookup ("activate",
 			       GTK_TYPE_MENU_ITEM);
-	if (sid != NULL) {
+	if (sid != 0) {
 	   g_signal_add_emission_hook (sid,
 				       0 /* detail */,
 				       delay_noblink,
@@ -280,7 +284,7 @@ gdm_common_setup_blinking (void)
 
 	sid = g_signal_lookup ("key_press_event",
 			       GTK_TYPE_WIDGET);
-	if (sid != NULL) {
+	if (sid != 0) {
 	   g_signal_add_emission_hook (sid,
 				       0 /* detail */,
 				       delay_noblink,
@@ -290,7 +294,7 @@ gdm_common_setup_blinking (void)
 
 	sid = g_signal_lookup ("button_press_event",
 			       GTK_TYPE_WIDGET);
-	if (sid != NULL) {
+	if (sid != 0) {
 	   g_signal_add_emission_hook (sid,
 				       0 /* detail */,
 				       delay_noblink,
@@ -570,6 +574,92 @@ pre_fetch_run (gpointer data)
 	return FALSE;
 }
 
+static gboolean 
+pre_atspi_launch (void){
+	gboolean a11y = gdm_config_get_bool (GDM_KEY_ADD_GTK_MODULES);
+	GPid pid = -1;
+	GError *error = NULL;
+	char *command = NULL;
+	gchar **atspi_prog_argv =  NULL;
+	
+	if (! a11y)
+		return FALSE;
+
+	command = g_strdup (LIBEXECDIR "/at-spi-registryd");
+
+	atspi_prog_argv = ve_split (command);
+	
+	g_spawn_async (".",
+		       atspi_prog_argv,
+		       NULL,
+		       (GSpawnFlags) (G_SPAWN_SEARCH_PATH),
+		       NULL,
+		       NULL,
+		       &pid,
+		       &error);
+
+	if (kill (pid, 0) < 0) {
+		fprintf (stderr, "at-spi-registryd not running: %s\n", error->message);
+		return FALSE;
+	}
+
+	return TRUE;
+
+}
+
+
+static GdkFilterReturn 
+filter_watch (GdkXEvent *xevent, GdkEvent *event, gpointer data){
+     XEvent *xev = (XEvent *)xevent;
+
+     if (xev->xany.type == PropertyNotify &&
+	 xev->xproperty.atom == AT_SPI_IOR){
+	     gtk_main_quit ();
+	  
+	  return GDK_FILTER_REMOVE;
+     }
+
+     return GDK_FILTER_CONTINUE;
+}
+
+static gboolean
+filter_timeout (gpointer data)
+{
+	gdm_common_info (_("The accessibility registry was not found."));
+
+	gtk_main_quit ();
+
+	return FALSE;
+}
+
+void
+gdm_common_atspi_launch (void)
+{
+	GdkWindow *w = gdk_get_default_root_window (); 
+	gboolean a11y = gdm_config_get_bool (GDM_KEY_ADD_GTK_MODULES);
+	guint tid;
+
+	if (! a11y)
+		return;
+ 
+	if ( ! AT_SPI_IOR)
+		AT_SPI_IOR = XInternAtom (GDK_DISPLAY (), "AT_SPI_IOR", False);
+
+	gdk_window_set_events (w,  GDK_PROPERTY_CHANGE_MASK); 
+	
+	if ( ! pre_atspi_launch ()) {
+		gdm_common_info (_("The accessibility registry could not be started."));
+		
+		return;
+	}
+	gdk_window_add_filter (w, filter_watch, NULL);
+	tid = g_timeout_add (1e3, filter_timeout, NULL);
+
+	gtk_main ();
+
+	g_source_remove (tid);
+}
+
 void
 gdm_common_pre_fetch_launch (void)
 {
